import os
import sys
import argparse

from django.core.management import execute_from_command_line

os.environ["DJANGO_SETTINGS_MODULE"] = "django_react_templatetags.tests.demosite.settings"


def runtests():
    args, rest = argparse.ArgumentParser().parse_known_args()

    argv = [sys.argv[0], "test"] + rest
    execute_from_command_line(argv)


if __name__ == "__main__":
    runtests()



# -*- coding: utf-8 -*-

"""
This module contains tags for including react components into templates.
"""
import uuid
import json

from django import template
from django.conf import settings
from django.template import Node
from django.utils.module_loading import import_string

from django_react_templatetags.encoders import json_encoder_cls_factory


register = template.Library()

CONTEXT_KEY = "REACT_COMPONENTS"

DEFAULT_SSR_HEADERS = {
    'Content-type': 'application/json',
    'Accept': 'text/plain',
}


def get_uuid():
    return uuid.uuid4().hex


def has_ssr(request):
    if request and request.META.get("HTTP_X_DISABLE_SSR"):
        return False

    return hasattr(settings, 'REACT_RENDER_HOST') and \
        settings.REACT_RENDER_HOST


def get_ssr_headers():
    if not hasattr(settings, 'REACT_RENDER_HEADERS'):
        return DEFAULT_SSR_HEADERS
    return settings.REACT_RENDER_HEADERS


def load_from_ssr(component, ssr_context=None):
    ssr_service = _get_ssr_service()()
    return ssr_service.load_or_empty(
        component,
        headers=get_ssr_headers(),
        ssr_context=ssr_context,
    )


def _get_ssr_service():
    """
    Loads a custom React Tag Manager if provided in Django Settings.
    """

    class_path = getattr(settings, 'REACT_SSR_SERVICE', '')
    if not class_path:
        from django_react_templatetags.ssr.default import SSRService
        return SSRService

    return import_string(class_path)


class ReactTagManager(Node):
    """
    Handles the printing of react placeholders and queueing, is invoked by
    react_render.
    """
    def __init__(
        self,
        identifier,
        component,
        data=None,
        css_class=None,
        props=None,
        ssr_context=None,
        no_placeholder=None
    ):
        component_prefix = ""
        if hasattr(settings, "REACT_COMPONENT_PREFIX"):
            component_prefix = settings.REACT_COMPONENT_PREFIX

        self.identifier = identifier
        self.component = component
        self.component_prefix = component_prefix
        self.data = data
        self.css_class = css_class
        self.props = props
        self.ssr_context = ssr_context
        self.no_placeholder= no_placeholder

    def render(self, context):
        qualified_component_name = self.get_qualified_name(context)
        identifier = self.get_identifier(context, qualified_component_name)
        component_props = self.get_component_props(context)
        json_str = self.props_to_json(component_props, context)

        component = {
            'identifier': identifier,
            'data_identifier': "{}_data".format(identifier),
            'name': qualified_component_name,
            'json': json_str,
            'json_obj': json.loads(json_str),
        }

        placeholder_attr = (
            ('id', identifier),
            ('class', self.resolve_template_variable(self.css_class, context)),
        )
        placeholder_attr = [x for x in placeholder_attr if x[1] is not None]

        component_html = ""
        if has_ssr(context.get("request", None)):
            ssr_resp = load_from_ssr(
                component,
                ssr_context=self.get_ssr_context(context),
            )
            component_html = ssr_resp["html"]
            component["ssr_params"] = ssr_resp["params"]

        components = context.get(CONTEXT_KEY, [])
        components.append(component)
        context[CONTEXT_KEY] = components

        if self.no_placeholder:
            return component_html

        return self.render_placeholder(placeholder_attr, component_html)

    def get_qualified_name(self, context):
        component_name = self.resolve_template_variable(self.component, context)
        return '{}{}'.format(self.component_prefix, component_name)

    def get_identifier(self, context, qualified_component_name):
        identifier = self.resolve_template_variable(self.identifier, context)

        if identifier:
            return identifier

        return '{}_{}'.format(qualified_component_name, get_uuid())

    def get_component_props(self, context):
        resolved_data = self.resolve_template_variable_else_none(self.data, context)
        resolved_data = resolved_data if resolved_data else {}

        for prop in self.props:
            data = self.resolve_template_variable_else_none(
                self.props[prop],
                context,
            )
            resolved_data[prop] = data

        return resolved_data

    def get_ssr_context(self, context):
        if not self.ssr_context:
            return {}

        return self.resolve_template_variable(self.ssr_context, context)

    @staticmethod
    def resolve_template_variable(value, context):
        if isinstance(value, template.Variable):
            return value.resolve(context)

        return value

    @staticmethod
    def resolve_template_variable_else_none(value, context):
        try:
            data = value.resolve(context)
        except template.VariableDoesNotExist:
            data = None
        except AttributeError:
            data = None

        return data

    @staticmethod
    def props_to_json(resolved_data, context):
        cls = json_encoder_cls_factory(context)
        return json.dumps(resolved_data, cls=cls)

    @staticmethod
    def render_placeholder(attributes, component_html=''):
        attr_pairs = map(lambda x: '{}="{}"'.format(*x), attributes)
        return u'<div {}>{}</div>'.format(
            " ".join(attr_pairs),
            component_html,
        )


@register.tag
def react_render(parser, token):
    """
    Renders a react placeholder and adds it to the global render queue.

    Example:
        {% react_render component="ListRestaurants" data=restaurants %}
    """

    values = _prepare_args(parser, token)
    tag_manager = _get_tag_manager()
    return tag_manager(**values)


def _prepare_args(parses, token):
    """
    Normalize token arguments that can be passed along to node renderer
    """

    values = {
        "identifier": None,
        "css_class": None,
        "data": None,
        "props": {},
    }

    key_mapping = {
        "id": "identifier",
        "class": "css_class",
        "props": "data",
    }

    args = token.split_contents()
    method = args[0]

    for arg in args[1:]:
        key, value = arg.split(r'=',)

        key = key_mapping.get(key, key)
        is_standalone_prop = key.startswith('prop_')
        if is_standalone_prop:
            key = key[5:]

        value = template.Variable(value)
        if is_standalone_prop:
            values['props'][key] = value
        else:
            values[key] = value

    assert "component" in values, "{} is missing component value".format(method)  # NOQA

    return values


def _get_tag_manager():
    """
    Loads a custom React Tag Manager if provided in Django Settings.
    """

    class_path = getattr(settings, 'REACT_RENDER_TAG_MANAGER', '')
    if not class_path:
        return ReactTagManager

    return import_string(class_path)


@register.inclusion_tag('react_print.html', takes_context=True)
def react_print(context):
    """
    Generates ReactDOM.hydate calls based on REACT_COMPONENT queue,
    this needs to be run after react has been loaded.

    The queue will be cleared after beeing called.

    Example:
        {% react_print %}
    """
    components = context.get(CONTEXT_KEY, [])
    context[CONTEXT_KEY] = []

    new_context = context.__copy__()
    new_context['ssr_available'] = has_ssr(
        context.get("request", None)
    )
    new_context['components'] = components

    return new_context



import re

import django
from django.template import Context, Template
from django.test import SimpleTestCase, override_settings
from django.test.client import RequestFactory

from django_react_templatetags.tests.demosite.models import Person, Movie


class ReactIncludeComponentTest(SimpleTestCase):
    def setUp(self):
        self.mocked_context = Context({'REACT_COMPONENTS': []})

    def test_react_tag(self):
        "The react_render inserts one components into the template"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<div id="Component_' in out)

    def test_multiple_tags(self):
        "The react_render inserts two components into the template"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<div id="Component_' in out)
        self.assertEquals(len(self.mocked_context.get("REACT_COMPONENTS")), 2)

    def test_component_name_from_variable(self):
        "The react_render inserts with a component id as a variable"

        self.mocked_context["component_name"] = 'DynamicComponentName'

        out = Template(
            "{% load react %}"
            "{% react_render component=component_name %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('React.createElement(DynamicComponentName' in out)

    def test_react_json_data_tag(self):
        "Tests that the data is added as correct json into the react render"

        self.mocked_context["component_data"] = {'name': 'Tom Waits'}

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" data=component_data %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('{\"name\": \"Tom Waits\"}' in out)

    @override_settings(
        REACT_COMPONENT_PREFIX="Components."
    )
    def test_react_component_prefix(self):
        "Tests that a prefix is added to the component createElement script"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<div id="Components.Component_' in out)

    def test_print_tag(self):
        "Makes sure the react_render gets emptied from context after print"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('ReactDOM.render(' in out)
        self.assertTrue('React.createElement(Component' in out)
        self.assertEquals(len(self.mocked_context.get("REACT_COMPONENTS")), 0)

    @override_settings(
        REACT_COMPONENT_PREFIX="ReactNamespace."
    )
    def test_print_tag_prefix(self):
        "Makes sure react_print outputs ReactDOM.render with react prefix"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('React.createElement(ReactNamespace.Component' in out)

    def test_variable_identifier(self):
        "Tests that the identifier variable is evaluated as variable"

        self.mocked_context["component_identifier"] = "TomWaits"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" identifier=component_identifier %}"  # NOQA
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('TomWaits' in out)

    def test_class_property(self):
        "Makes sure class property are applied"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" class=\"component-class\" %}"  # NOQA
        ).render(self.mocked_context)

        self.assertTrue('class="component-class"' in out)

    def test_unresolved_props_wont_raise_error(self):
        "Makes sure class property are applied"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" props=nonexisting_val %}"  # NOQA
        ).render(self.mocked_context)

        self.assertTrue('<div id="' in out)

    def test_class_property_from_variable(self):
        self.mocked_context["class_name"] = "component-class"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" class=class_name %}"  # NOQA
        ).render(self.mocked_context)

        self.assertTrue('class="component-class"' in out)

    def test_model_representation_data(self):
        "Tests that react representation of model is transformed"

        person = Person(first_name='Tom', last_name='Waits')

        self.mocked_context["component_data"] = person

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" data=component_data %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('\"first_name\": \"Tom\"' in out)
        self.assertTrue('\"last_name\": \"Waits\"' in out)

    def test_no_represntation_mixin_raises_error(self):
        if django.VERSION < (3, 0):
            "Skip this test in django 2 due to different output format"
            return

        class NoRepresentation(object):
            pass

        instance = NoRepresentation()
        self.mocked_context["component_data"] = instance

        with self.assertRaises(TypeError) as err:
            out = Template(
                "{% load react %}"
                "{% react_render component=\"Component\" data=component_data %}"
                "{% react_print %}"
            ).render(self.mocked_context)

        self.assertTrue(
            re.search(
                r'Object of type \'{,1}NoRepresentation\'{,1} is not JSON serializable',
                str(err.exception)
            )
        )

    def test_individual_prop_data(self):
        "Tests that templatetag can accept individual prop types"

        person = Person(first_name='Tom', last_name='Waits')

        self.mocked_context["person"] = person
        self.mocked_context["album"] = 'Small Change'

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_person=person prop_album=album %}"  # NOQA
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('person\":' in out)
        self.assertFalse('prop_person":' in out)
        self.assertTrue('\"first_name\": \"Tom\"' in out)
        self.assertTrue('\"last_name\": \"Waits\"' in out)
        self.assertTrue('album\": \"Small Change\"' in out)

    def test_combined_data_and_individual_props(self):
        "Tests that templatetag can accept individual prop types"

        person = Person(first_name='Tom', last_name='Waits')

        self.mocked_context["person"] = person
        self.mocked_context["component_data"] = {'name': 'Tom Waits'}

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" data=component_data prop_person=person %}"  # NOQA
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('person\":' in out)
        self.assertTrue('\"first_name\": \"Tom\"' in out)
        self.assertTrue('\"last_name\": \"Waits\"' in out)
        self.assertTrue('name\": \"Tom Waits\"' in out)

    def test_support_for_props_data_fallback_arg(self):
        "Tests that templatetag can accept individual prop types"

        self.mocked_context["component_data"] = {'name': 'Tom Waits'}

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" props=component_data %}"  # NOQA
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('\"name\": \"Tom Waits\"' in out)

    def test_to_model_representation_data(self):
        "Tests that to_react_representation renders proper json"

        movie = Movie(title='Night On Earth', year=1991)

        self.mocked_context["component_data"] = movie
        self.mocked_context["request"] = RequestFactory().get('/random')

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" data=component_data %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('\"title\": \"Night On Earth\"' in out)
        self.assertTrue('\"year\": 1991' in out)
        self.assertTrue('\"current_path\": \"/random\"' in out)

    def test_unicode_chars(self):
        "Tests that the data is added as correct json into the react render"

        self.mocked_context["component_data"] = {'name': u'ÅÄÖ'}

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" data=component_data %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('{\"name\": \"\\u00c5\\u00c4\\u00d6\"}' in out)

    def test_prop_strings_not_null(self):
        "Test that standalone string props are not returned as null"

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_country=\"Sweden\" %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('{\"country\": \"Sweden\"}' in out)

    def test_no_placeholder_returns_nothing(self):
        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_country=\"Sweden\" no_placeholder=1 %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertFalse(out.startswith('<div id="Component_'))
        self.assertTrue('{\"country\": \"Sweden\"}' in out)

    def test_data_script_is_included(self):
        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" identifier=\"my_id\" %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertFalse(out.startswith('<script id="my_id_data" type="application/json"'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
django_react_templatetags
----------
This extension allows you to add React components into your django templates.
"""

__title__ = "django_react_templatetags"
__version__ = "6.0.1"
__build__ = 600
__author__ = "Martin Sandström"
__license__ = "MIT"
__copyright__ = "Copyright 2015-2020 Fröjd Interactive"



try:
    from unittest import mock
except ImportError:
    import mock

from django.urls import reverse
from django.template import Context, Template
from django.test import SimpleTestCase, override_settings

from django_react_templatetags.tests.demosite.models import (
    Person, MovieWithContext
)


class CustomSSRService():
    "Used for testing using a custom ssr service"
    def load_or_empty(self, component, headers={}, ssr_context=None):
        pass


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/',
    REACT_SSR_SERVICE="django_react_templatetags.tests.test_ssr.CustomSSRService",
)
class CustomSSRServiceTest(SimpleTestCase):
    @mock.patch("django_react_templatetags.tests.test_ssr.CustomSSRService.load_or_empty")
    def test_that_disable_ssr_header_disables_ssr(self, mocked_func):
        self.client.get(
            reverse('static_react_view'),
        )
        self.assertEqual(mocked_func.call_count, 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.serializers.json import DjangoJSONEncoder
from django_react_templatetags.mixins import RepresentationMixin


def json_encoder_cls_factory(context):
    class ReqReactRepresentationJSONEncoder(ReactRepresentationJSONEncoder):
        context = None

    ReqReactRepresentationJSONEncoder.context = context
    return ReqReactRepresentationJSONEncoder


class ReactRepresentationJSONEncoder(DjangoJSONEncoder):
    '''
    Custom json encoder that adds support for RepresentationMixin
    '''

    def default(self, o):
        if isinstance(o, RepresentationMixin):
            args = [self.context if hasattr(self, 'context') else None]
            args = [x for x in args if x is not None]

            return o.to_react_representation(*args)

        return super(ReactRepresentationJSONEncoder, self).default(o)



from django.conf.urls import url


from django_react_templatetags.tests.demosite import views


urlpatterns = [
    url(
        'static-react-view',
        views.StaticReactView.as_view(),
        name='static_react_view',
    ),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import warnings


def react_context_processor(request):
    """Expose a global list of react components to be processed"""
    warnings.warn(
        "react_context_processor is no longer required.", DeprecationWarning
    )

    return {
        'REACT_COMPONENTS': [],
    }



# -*- coding: utf-8 -*-

"""
This modules manages SSR rendering logic
"""

import logging

from django.conf import settings
import requests
import json


logger = logging.getLogger(__name__)


class SSRService():
    def load_or_empty(self, component, headers={}, ssr_context=None):
        request_json = u'{{"componentName": "{0}", "props": {1}, "context": {2}}}'.format(
            component['name'],
            component['json'],
            json.dumps(ssr_context) if ssr_context else {},
        )

        try:
            inner_html = self.load(request_json, headers)
        except requests.exceptions.RequestException as e:
            inner_html = ''

            msg = "SSR request to '{}' failed: {}".format(
                settings.REACT_RENDER_HOST,
                e.__class__.__name__
            )
            logger.exception(msg)

        return {
            "html": inner_html,
            "params": {},
        }


    def load(self, request_json, headers):
        req = requests.post(
            settings.REACT_RENDER_HOST,
            timeout=get_request_timeout(),
            data=request_json,
            headers=headers,
        )

        req.raise_for_status()
        return req.text


def get_request_timeout():
    if not hasattr(settings, 'REACT_RENDER_TIMEOUT'):
        return 20

    return settings.REACT_RENDER_TIMEOUT



from django.views.generic import TemplateView


class StaticReactView(TemplateView):
    template_name = "static-react.html"

    def get_context_data(self, **kwargs):
        context = super(StaticReactView, self).get_context_data(**kwargs)
        context["props"] = {
            "artist": "Tom Waits",
            "recent_album": "Bad as me",
        }
        return context



import logging
import json
import re

from django.conf import settings
import hypernova


logger = logging.getLogger(__name__)
hypernova_id_re = re.compile(r"data-hypernova-id=\"([\w\-]*)\"")
hypernova_key_re = re.compile(r"data-hypernova-key=\"([\w\-]*)\"")


class HypernovaService:
    def load_or_empty(self, component, headers={}, ssr_context=None):
        # from hypernova.plugins.dev_mode import DevModePlugin

        renderer = hypernova.Renderer(
            settings.REACT_RENDER_HOST,
            # [DevModePlugin(logger)] if settings.DEBUG else [],
            [],
            timeout=get_request_timeout(),
            headers=headers,
        )

        inner_html = ""
        try:
            props = component["json_obj"]
            if ssr_context:
                props["context"] = ssr_context
            inner_html = renderer.render({component["name"]: props})
        except Exception as e:
            msg = "SSR request to '{}' failed: {}".format(
                settings.REACT_RENDER_HOST, e.__class__.__name__
            )
            logger.exception(msg)

        if not inner_html:
            return {"html": "", "params": {}}

        match = re.search(hypernova_id_re, inner_html)
        hypernova_id = match.group(1) if match else None

        match = re.search(hypernova_key_re, inner_html)
        hypernova_key = match.group(1) if match else None

        return {
            "html": inner_html,
            "params": {"hypernova_id": hypernova_id, "hypernova_key": hypernova_key,},
        }


def get_request_timeout():
    if not hasattr(settings, "REACT_RENDER_TIMEOUT"):
        return 20

    return settings.REACT_RENDER_TIMEOUT



class MockResponse:
    def __init__(self, data, status_code, ok=True):
        self.data = data
        self.text = data
        self.status_code = status_code
        self.ok = ok

    def raise_for_status(self):
        pass

    def json(self):
        return self.data



from django.test import SimpleTestCase, override_settings

from django_react_templatetags.mixins import RepresentationMixin


class RepresentationMixinTest(SimpleTestCase):
    def test_implementation_error_raised_if_not_fully_implemented(self):
        class MyObj(RepresentationMixin, object):
            pass

        instance = MyObj()

        with self.assertRaises(NotImplementedError) as err:
            instance.to_react_representation()

        self.assertEquals(
            str(err.exception),
            "Missing property to_react_representation in class"
        )




import json

try:
    from unittest import mock
except ImportError:
    import mock

from django.urls import reverse
from django.template import Context, Template
from django.test import SimpleTestCase, override_settings

from .mock_response import MockResponse
from django_react_templatetags.tests.demosite.models import (
    Person, MovieWithContext
)
from django_react_templatetags.ssr.default import SSRService


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/',
)
class SSRTemplateTest(SimpleTestCase):
    def setUp(self):
        self.mocked_context = Context({'REACT_COMPONENTS': []})

    @mock.patch('requests.post')
    def test_verify_404(self, mocked):
        "The SSR rendering falls back to client side rendering if 404"

        mocked.side_effect = [MockResponse({'error': 'not found'}, 404)]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<div id="Component_' in out)

    @mock.patch('requests.post')
    def test_that_only_html_resp_are_shown_in_template(self, mocked):
        mocked.side_effect = [MockResponse('<h1>Title</h1>', 200)]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertFalse("{'html': " in out)

    @mock.patch('requests.post')
    def test_verify_rendition(self, mocked):
        "The SSR returns inner html"

        mocked.side_effect = [MockResponse('<h1>Title</h1>', 200)]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<h1>Title</h1>' in out)

    @mock.patch('requests.post')
    def test_request_body(self, mocked):
        "The SSR request sends the props in a expected way"

        mocked.side_effect = [MockResponse('<h1>Title</h1>', 200)]

        person = Person(first_name='Tom', last_name='Waits')

        self.mocked_context["person"] = person
        self.mocked_context["component_data"] = {'album': 'Real gone'}

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_person=person data=component_data %}"  # NOQA
        ).render(self.mocked_context)

        request_body = {
            'componentName': 'Component',
            'props': {
                'album': 'Real gone',
                'person': {
                    'first_name': 'Tom',
                    'last_name': 'Waits'
                }
            },
            'context': {}
        }

        self.assertEqual(json.loads(mocked.call_args[1]["data"]), request_body)

    @mock.patch('requests.post')
    def test_request_body_context(self, mocked):
        "The SSR request sends the props in a expected way with context"

        mocked.side_effect = [MockResponse('<h1>Title</h1>', 200)]

        movie = MovieWithContext(title='Office space', year=1991)

        self.mocked_context["movie"] = movie
        self.mocked_context["search_term"] = 'Stapler'

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_movie=movie %}"
        ).render(self.mocked_context)

        request_body = {
            'componentName': 'Component',
            'props': {
                'movie': {
                    'title': 'Office space',
                    'year': 1991,
                    'search_term': 'Stapler',
                }
            },
            'context': {}
        }

        self.assertEqual(json.loads(mocked.call_args[1]["data"]), request_body)

    @mock.patch('requests.post')
    def test_request_body_with_ssr_context(self, mocked):
        "The SSR request appends the 'ssr_context' in an expected way"

        mocked.side_effect = [MockResponse('<h1>Title</h1>', 200)]

        self.mocked_context["ssr_ctx"] = {"location": "http://localhost"}

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" ssr_context=ssr_ctx %}"
        ).render(self.mocked_context)

        request_body = {
            'componentName': 'Component',
            'props': {},
            'context': {'location': "http://localhost"}
        }

        self.assertEqual(json.loads(mocked.call_args[1]["data"]), request_body)

    @mock.patch('requests.post')
    def test_default_headers(self, mocked):
        "The SSR uses default headers with json as conten type"
        mocked.side_effect = [MockResponse('Foo Bar', 200)]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        headers = {
            'Content-type': 'application/json',
            'Accept': 'text/plain',
        }

        self.assertEqual(mocked.call_count, 1)
        self.assertEqual(mocked.call_args[1]["headers"], headers)

    @override_settings(
        REACT_RENDER_HEADERS={
            'Authorization': 'Basic 123'
        }
    )
    @mock.patch('requests.post')
    def test_custom_headers(self, mocked):
        "The SSR uses custom headers if present"
        mocked.side_effect = [MockResponse('Foo Bar', 200)]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue(mocked.call_count == 1)
        self.assertEqual(mocked.call_args[1]["headers"]['Authorization'], 'Basic 123')

    @mock.patch('requests.post')
    def test_hydrate_if_ssr_present(self, mocked):
        "Makes sure ReactDOM.hydrate is used when SSR is active"
        mocked.side_effect = [MockResponse('Foo Bar', 200)]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
            "{% react_print %}"
        ).render(self.mocked_context)

        self.assertTrue('ReactDOM.hydrate(' in out)

    @mock.patch('requests.post')
    def test_ssr_params_are_stored_in_component_queue(self, mocked):
        mocked.side_effect = [MockResponse('Foo Bar', 200)]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        queue = self.mocked_context["REACT_COMPONENTS"]
        self.assertTrue("ssr_params" in queue[0])
        self.assertEqual(queue[0]["ssr_params"], {})


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/',
)
class SSRViewTest(SimpleTestCase):
    @mock.patch("django_react_templatetags.ssr.default.SSRService.load_or_empty")
    def test_that_disable_ssr_header_disables_ssr(self, mocked_func):
        self.client.get(
            reverse('static_react_view'),
            HTTP_X_DISABLE_SSR='1',
        )
        self.assertEqual(mocked_func.call_count, 0)


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/batch',
)
class DefaultServiceTest(SimpleTestCase):
    @mock.patch('requests.post')
    def test_load_or_empty_returns_ok_data(self, mocked):
        mocked.side_effect = [MockResponse('Foo Bar', 200)]

        service = SSRService()
        resp = service.load_or_empty({
            "json": "{}",
            "name": "App",
        })
        self.assertTrue("html" in resp)
        self.assertTrue("Foo Bar" in resp["html"])
        self.assertTrue("params" in resp)
        params = resp["params"]
        self.assertEqual(params, {})



#!/usr/bin/env python

import os


PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

DEBUG = False

TIME_ZONE = "Europe/Stockholm"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

SECRET_KEY = "not needed"

USE_TZ = True

LANGUAGE_CODE = "en"

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sites",
    "django.contrib.admin",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django_react_templatetags",
    "django_react_templatetags.tests.demosite",
]

ROOT_URLCONF = "django_react_templatetags.tests.demosite.urls"

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]



from __future__ import unicode_literals

from django.db import models

from django_react_templatetags.mixins import RepresentationMixin


class Person(RepresentationMixin, models.Model):
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)

    def to_react_representation(self, context={}):
        return {
            'first_name': self.first_name,
            'last_name': self.last_name,
        }


class Movie(RepresentationMixin, models.Model):
    title = models.CharField(max_length=255)
    year = models.IntegerField()

    def to_react_representation(self, context={}):
        return {
            'title': self.title,
            'year': self.year,
            'current_path': context['request'].path,
        }


class MovieWithContext(RepresentationMixin, models.Model):
    title = models.CharField(max_length=255)
    year = models.IntegerField()

    def to_react_representation(self, context={}):
        return {
            'title': self.title,
            'year': self.year,
            'search_term': context['search_term'],
        }



# -*- coding: utf-8 -*-

from django.template import Context, Template
from django.test import SimpleTestCase, override_settings

from django_react_templatetags.templatetags.react import (
    _get_tag_manager,
    ReactTagManager,
)


class TestReactTagManager(ReactTagManager):
    def render(self, context):
        return 'Test'


class ReactIncludeComponentTest(SimpleTestCase):
    def setUp(self):
        self.mocked_context = Context({'REACT_COMPONENTS': []})

    def test_tag_manager_not_overridden(self):
        "Test that the default ReactTagManager is used by default"
        self.assertEqual(_get_tag_manager(), ReactTagManager)

    @override_settings(
        REACT_RENDER_TAG_MANAGER="django_react_templatetags.tests.test_manager.TestReactTagManager"
    )
    def test_tag_manager_overridden(self):
        "Test that the TestReactTagManager is actually used"
        self.assertEqual(_get_tag_manager(), TestReactTagManager)

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertEqual('Test', out)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys
import re
from setuptools import setup, find_packages


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()


with open('README.md') as f:
    readme = f.read()

# Convert markdown to rst
try:
    from pypandoc import convert_file
    long_description = convert_file("README.md", "rst")
except:
    long_description = ""

version = ''
with io.open('django_react_templatetags/__init__.py', 'r', encoding='utf8') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

setup(
    name="django_react_templatetags",
    version=version,
    description=("This django library allows you to add React components into your django templates."),  # NOQA
    long_description=long_description,
    author="Fröjd",
    author_email="martin@marteinn.se",
    url="https://github.com/frojd/django-react-templatetags",
    packages=find_packages(exclude=('tests*', "example_django_react_templatetags")),
    include_package_data=True,
    install_requires=[
        'Django>=1.11',
    ],
    extras_require={
        'ssr': ['requests'],
        'hypernova': ['hypernova'],
    },
    tests_require=[
        'Django>=1.11',
        'requests',
    ],
    license="MIT",
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Environment :: Web Environment',
        "Intended Audience :: Developers",
        "Natural Language :: English",
        'Intended Audience :: Developers',
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2',
        "Programming Language :: Python :: 2.7",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Topic :: Utilities',
        'Programming Language :: JavaScript',
    ],
)



import json
import logging

from .mock_response import MockResponse

try:
    from unittest import mock
except ImportError:
    import mock

from django.template import Context, Template
from django.test import SimpleTestCase, override_settings

from django_react_templatetags.tests.demosite.models import (
    Person, MovieWithContext
)
from django_react_templatetags.ssr.hypernova import HypernovaService


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/batch',
    REACT_SSR_SERVICE="django_react_templatetags.ssr.hypernova.HypernovaService",
)
class HypernovaTemplateTest(SimpleTestCase):
    def setUp(self):
        self.mocked_context = Context({'REACT_COMPONENTS': []})

    @mock.patch('requests.post')
    def test_verify_404(self, mocked):
        "The SSR rendering falls back to client side rendering if 404"

        mocked.side_effect = [
            MockResponse({'error': 'not found'}, 404, ok=False)
        ]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<div id="Component_' in out)

    @mock.patch('requests.post')
    def test_that_only_html_resp_are_shown_in_template(self, mocked):
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('<h1>Title</h1>'),
                200,
            )
        ]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertFalse("{'html': " in out)

    @mock.patch('requests.post')
    def test_verify_rendition(self, mocked):
        "The SSR returns inner html"

        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('<h1>Title</h1>'),
                200,
            )
        ]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue('<h1>Title</h1>' in out)

    @mock.patch('requests.post')
    def test_request_body(self, mocked):
        "The SSR request sends the props in a expected way"

        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('<h1>Title</h1>'),
                200,
            )
        ]

        person = Person(first_name='Tom', last_name='Waits')

        self.mocked_context["person"] = person
        self.mocked_context["component_data"] = {'album': 'Real gone'}

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_person=person data=component_data %}"  # NOQA
        ).render(self.mocked_context)

        request_body = {
            "album": "Real gone",
            "person": {
                "first_name": "Tom",
                "last_name": "Waits"
            }
        }
        self.assertTrue('Component' in mocked.call_args[1]["json"])
        self.assertEqual(
            mocked.call_args[1]["json"]["Component"]['data'],
            request_body
        )

    @mock.patch('requests.post')
    def test_request_body_context(self, mocked):
        "The SSR request sends the props in a expected way with context"

        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('<h1>Title</h1>'),
                200,
            )
        ]

        movie = MovieWithContext(title='Office space', year=1991)

        self.mocked_context["movie"] = movie
        self.mocked_context["search_term"] = 'Stapler'

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" prop_movie=movie %}"
        ).render(self.mocked_context)

        request_body = {
            "movie": {
                "title": "Office space",
                "year": 1991,
                "search_term": "Stapler",
            }
        }

        self.assertTrue('Component' in mocked.call_args[1]["json"])
        self.assertEqual(
            mocked.call_args[1]["json"]["Component"]['data'],
            request_body
        )

    @mock.patch('requests.post')
    def test_default_headers(self, mocked):
        "The SSR uses default headers with json as conten type"
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar'),
                200,
            )
        ]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        headers = {
            'Content-type': 'application/json',
            'Accept': 'text/plain',
        }

        self.assertEqual(mocked.call_count, 1)
        self.assertEqual(mocked.call_args[1]["headers"], headers)

    @override_settings(
        REACT_RENDER_HEADERS={
            'Authorization': 'Basic 123'
        }
    )
    @mock.patch('requests.post')
    def test_custom_headers(self, mocked):
        "The SSR uses custom headers if present"
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar'),
                200,
            )
        ]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        self.assertTrue(mocked.call_count == 1)
        self.assertEqual(mocked.call_args[1]["headers"]['Authorization'], 'Basic 123')

    @mock.patch('requests.post')
    def test_ssr_params_are_stored_in_component_queue(self, mocked):
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar'),
                200,
            )
        ]

        Template(
            "{% load react %}"
            "{% react_render component=\"Component\" %}"
        ).render(self.mocked_context)

        queue = self.mocked_context["REACT_COMPONENTS"]
        self.assertTrue("ssr_params" in queue[0])

        ssr_params = queue[0]["ssr_params"]
        self.assertIn("hypernova_id", ssr_params)
        self.assertIn("hypernova_key", ssr_params)

    @mock.patch('requests.post')
    def test_only_ssr_html_are_returned_on_no_placeholder(self, mocked):
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar'),
                200,
            )
        ]

        out = Template(
            "{% load react %}"
            "{% react_render component=\"Component\" no_placeholder=1 %}"
        ).render(self.mocked_context)

        queue = self.mocked_context["REACT_COMPONENTS"]
        self.assertTrue(len(queue), 1)
        self.assertFalse(out.startswith('<div id="Component_'))


@override_settings(
    REACT_RENDER_HOST='http://react-service.dev/batch',
    REACT_SSR_SERVICE="django_react_templatetags.ssr.hypernova.HypernovaService",
)
class HypernovaServiceTest(SimpleTestCase):
    @mock.patch('requests.post')
    def test_load_or_empty_returns_ok_data(self, mocked):
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar', id="my-id", key="App"),
                200,
            )
        ]

        service = HypernovaService()
        resp = service.load_or_empty({
            "json_obj": {},
            "name": "App",
        })
        self.assertTrue("html" in resp)
        self.assertTrue("Foo Bar" in resp["html"])
        self.assertTrue("params" in resp)
        params = resp["params"]
        self.assertTrue("hypernova_id" in params)
        self.assertEqual(params["hypernova_id"], "my-id")
        self.assertTrue("hypernova_key" in params)
        self.assertEqual(params["hypernova_key"], "App")

    @mock.patch('requests.post')
    def test_ssr_context_are_passed_to_hypernova_as_prop(self, mocked):
        mocked.side_effect = [
            MockResponse(
                mock_hypernova_success_response('Foo Bar', id="my-id", key="App"),
                200,
            )
        ]

        service = HypernovaService()
        service.load_or_empty(
            {
                "json_obj": {"month": 4},
                "name": "App",
            },
            ssr_context={
                "language": "en",
            }
        )

        post_data = mocked.call_args[1]["json"]
        self.assertIn("language", post_data["App"]["data"]["context"])
        self.assertEqual(post_data["App"]["data"]["context"]["language"], "en")

    @mock.patch('requests.post')
    def test_empty_html_are_returned_on_request_error(self, mocked):
        logging.disable(logging.CRITICAL)

        mocked.side_effect = [
            MockResponse("", 500)
        ]

        service = HypernovaService()
        resp = service.load_or_empty({
            "json_obj": {},
            "name": "App",
        })

        self.assertEqual(resp["html"], "")
        self.assertEqual(resp["params"], {})

        logging.disable(logging.NOTSET)


def mock_hypernova_success_response(
    body, component_name="App", id="novaid-1", key="Appjs"
):
    html = "<div data-hypernova-key=\"{}\" data-hypernova-id=\"{}\">" \
        "{}".format(key, id, body) + \
        "</div>\n" + \
        "<script type=\"application/json\" data-hypernova-key=\"{}\" data-hypernova-id=\"{}\">".format(key, id) + \
        "<!--{}--></script>"  # NOQA

    return {
        "success": True,
        "error": None,
        "results": {
            component_name: {
                "name": component_name,
                "html": html,
                "meta": {},
                "duration": 0.279824,
                "statusCode": 200,
                "success": True,
                "error": None,
            }
        }
    }
